require 'addressable/uri'
require 'shadow_cabinet/client/result'

module ShadowCabinet
  module Client
    class Base
      attr_reader :url, :key, :secret

      def initialize(url:, key:, secret:)
        @url = url
        @key = key
        @secret = secret
      end

      def get(path:, params: {})
        result = make_request(
          verb: :get,
          request_url: construct_url(path: path, params: params)
        )

        raise result.error if result.error_present?

        result.body
      end

      def post(path:, params: {}, data: nil)
        result = make_request(
          verb: :post,
          request_url: construct_url(path: path, params: params),
          data: data
        )

        raise result.error if result.error_present?

        result.body
      end

      def put(path:, params: {}, data: nil)
        result = make_request(
          verb: :put,
          request_url: construct_url(path: path, params: params),
          data: data
        )

        raise result.error if result.error_present?

        result.body
      end

      def patch(path:, params: {}, data: nil)
        result = make_request(
          verb: :patch,
          request_url: construct_url(path: path, params: params),
          data: data
        )

        raise result.error if result.error_present?

        result.body
      end

      def delete(path:, params: {})
        result = make_request(
          verb: :delete,
          request_url: construct_url(path, params)
        )

        raise result.error if result.error_present?

        result.body
      end

      private

      def make_request(verb:, request_url:, data: nil)
        Result.new(
          error: Unimplmented.new(
            "#make_request is not defined for #{self.class}"
          )
        )
      end

      def base_url
        @base_url ||= Addressable::URI.parse(url)
      end

      def construct_url(path:, params: {})
        base_url.tap do |url|
          url.path = File.join('/', 'api', 'v1', path)

          url.query_values = params

          return url
        end
      end

    end
  end
end
